/*
 *  Copyright  Northwoods Software Corporation, 2000-2007. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 *
 */

package com.nwoods.jgo.examples;

import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import com.nwoods.jgo.*;

/**
 * This is a minimal JGo applet/application. Most of the code here
 * is standard boilerplate that must be in any applet/application.
 * <p>
 * This just displays two BasicNodes, of different colors.
 * The user can only select nodes, move them, copy them, and link them together.
 */
public class MinimalApp {
  public MinimalApp(Shell shell) {
    myView = new JGoView(shell, SWT.V_SCROLL | SWT.H_SCROLL);
  }

  public static void main(String args[]) {
    try {
      Display display = new Display();
      Shell shell = new Shell(display);
      shell.setText("Minimal JGo Application");
      FillLayout layout = new FillLayout();
      layout.type = SWT.VERTICAL;
      shell.setLayout(layout);
      
      new MinimalApp(shell).init();

      shell.open();
      while (!shell.isDisposed()) {
        if (!display.readAndDispatch())
          display.sleep();
      }
    } catch (Throwable t) {
      System.err.println(t);
      t.printStackTrace();
      System.exit(1);
    }
  }

  public void init() {  // App initialization--only here do we do anything JGo specific
    // add JGoObjects to the document, not to the view
    JGoDocument doc = myView.getDocument();

    // create two nodes for fun...
    JGoBasicNode node1 = new JGoBasicNode("first");
    node1.setLocation(new Point(100, 100));  // specify position
    node1.setBrush(JGoBrush.blue);  // specify color
    doc.addObjectAtTail(node1);              // add to the document

    JGoBasicNode node2 = new JGoBasicNode("second");
    node2.setLocation(new Point(200, 100));
    node2.setBrush(JGoBrush.magenta);
    doc.addObjectAtTail(node2);
  }

  private JGoView myView;
}
