/*
 *  Copyright  Northwoods Software Corporation, 1998-2007. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 *
 */

package com.nwoods.jgo.examples.demo1SWT;

import java.util.*;
import org.eclipse.swt.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.dnd.*;
import java.io.*;
import java.lang.reflect.*;
import com.nwoods.jgo.*;
import com.nwoods.jgo.svg.*;
import com.nwoods.jgo.layout.*;
import com.nwoods.jgo.examples.*;

/**
 * A simple application demonstrating many features of JGo for SWT.
 */
public class Demo1 {
  /**
   * Constructor for the Demo1 sample application
   */
  public Demo1(Shell shell) {
    //==============================================================
    // Create the main graphics window
    //==============================================================
    myShell = shell;
    SashForm form = new SashForm(shell, SWT.HORIZONTAL);

    SashForm form2 = new SashForm(form, SWT.VERTICAL);
    myPalette = new JGoPalette(form2, SWT.LEFT | SWT.V_SCROLL | SWT.H_SCROLL);
    myList = new org.eclipse.swt.widgets.List(form2, SWT.V_SCROLL | SWT.SINGLE);
    myList.add("one");
    myList.add("two");
    myList.add("three");

    myView = new Demo1View(form, SWT.TOP | SWT.V_SCROLL | SWT.H_SCROLL);
    myView.setFocus();
    myDoc = myView.getDocument();
    myDoc.setMaintainsPartID(true);
    myView.setBackgroundColor(new RGB(0xFF, 0xFF, 0xDD));

    int[] weights = new int[2];
    weights[0] = 15;
    weights[1] = 85;
    form.setWeights(weights);

    Transfer[] myListTypes = new Transfer[] { TextTransfer.getInstance() };
    DragSource myListDragSource = new DragSource(myList, DND.DROP_COPY);
    myListDragSource.setTransfer(myListTypes);
    myListDragSource.addDragListener(new DragSourceAdapter() {
      public void dragStart(DragSourceEvent e) {
        if (myList.getSelectionIndex() < 0)
          e.doit = false;
      }
      public void dragSetData(DragSourceEvent e) {
        if (myList.getSelectionIndex() >= 0)
          e.data = myList.getItem(myList.getSelectionIndex());
      }
    });

    initMenus();

    initCanvas();

    initPalette();

    //==============================================================
    // Add event listeners
    //==============================================================

    myView.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent evt) {
        char ch = evt.character;
        int code = evt.keyCode;
        boolean ctrl = ((evt.stateMask & SWT.CTRL) != 0);
        boolean shift = ((evt.stateMask & SWT.SHIFT) != 0);
        if (code == SWT.F6) {
          if (ctrl) {
            if (shift) {
              myView.setScale(1);
            } else {
              myView.setScale(myView.getScale() * 1.1);
            }
          } else {
            if (shift) {
              myView.setScale(myView.getScale() / 1.1);
            } else {
            }
          }
        } else if (code == SWT.DEL) {
          myView.deleteSelection();
        } else if (code == SWT.F4) {
          JGoObject obj = myView.getSelection().getPrimarySelection();
          callDialog(obj);
        } else if (ctrl && ch == 1) {  // Ctrl-A
          myView.selectAll();
        } else if (ctrl && ch == 3) {  // Ctrl-C
          myView.copy();
        } else if (ctrl && ch == 17) { // Ctrl-Q
          System.exit(0);
        } else if (ctrl && ch == 16) { // Ctrl-P
          myView.print();
        } else if (ctrl && ch == 24) { // Ctrl-X
          myView.cut();
        } else if (ctrl && ch == 22) { // Ctrl-V
          myView.paste();
        } else if (ctrl && ch == 25) { // Ctrl-Y
          myView.getDocument().redo();
        } else if (ctrl && ch == 26) { // Ctrl-Z
          myView.getDocument().undo();
        }
      }
    });

    // Note: An alternate way to get notification of events from a JGoDocument
    // is to create a subclass of JGoView and override the documentChanged() method.
    myDoc.addDocumentListener(new JGoDocumentListener() {
      public void documentChanged(JGoDocumentEvent e) {
        processDocChange(e);
      }
    });

    // This will handle selection changes and double clicks
    // Note: because this demo application also defines a JGoView subclass,
    // we could override method(s) in Demo1View instead of putting
    // event handling code in a JGoViewListener.
    myView.addViewListener(new JGoViewListener() {
      public void viewChanged(JGoViewEvent e) {
        processViewChange(e);
      }
    });

    // turn on undo/redo
    myDoc.setUndoManager(new JGoUndoManager());
  }

  void initMenus() {
    //==============================================================
    // Set up menu bar
    //==============================================================
    // Create menu bar
    Menu mb = new Menu(myShell, SWT.BAR);
    
    // Create File menu
    MenuItem miFile = new MenuItem(mb, SWT.CASCADE);
    miFile.setText("&File");
    Menu fileMenu = new Menu(myShell, SWT.DROP_DOWN);
    miFile.setMenu(fileMenu);
    
    // Create File menu items
    MenuItem miSVG = new MenuItem(fileMenu, SWT.NONE);
    miSVG.setText("Save SVG");
    miSVG.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        FileOutputStream fstream = null;
        try {
          fstream = new FileOutputStream("output.svg");
          DefaultDocument svgDomDoc = new DefaultDocument();
          svgDomDoc.setGenerateJGoXML(true);
          svgDomDoc.setGenerateSVG(true);
          svgDomDoc.SVGWriteDoc(fstream, myDoc);
        } catch (Exception ex) {
          if (fstream != null) {
            try {
              fstream.close();
            } catch (Exception ex2) {}
          }
        }
      }
    });
    
    MenuItem miPrint = new MenuItem(fileMenu, SWT.NONE);
    miPrint.setText("&Print\tCtrl-P");
    miPrint.setAccelerator(SWT.CONTROL | 'P');
    miPrint.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.print();
      }
    });
    
    new MenuItem(fileMenu, SWT.SEPARATOR);
    
    MenuItem miExit = new MenuItem(fileMenu, SWT.NONE);
    miExit.setText("E&xit\tCtrl-Q");
    miExit.setAccelerator(SWT.CONTROL | 'Q');
    miExit.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myShell.close(); // calls dispose() - see note below
      }
    });
    
    // Create Edit menu
    MenuItem miEdit = new MenuItem(mb, SWT.CASCADE);
    miEdit.setText("&Edit");
    Menu editMenu = new Menu(myShell, SWT.DROP_DOWN);
    miEdit.setMenu(editMenu);
    
    // Create Edit menu items
    MenuItem miCut = new MenuItem(editMenu, SWT.NONE);
    miCut.setText("Cu&t\tCtrl-X");
    miCut.setAccelerator(SWT.CONTROL | 'X');
    miCut.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.cut();
      }
    });
    MenuItem miCopy = new MenuItem(editMenu, SWT.NONE);
    miCopy.setText("&Copy\tCtrl-C");
    miCopy.setAccelerator(SWT.CONTROL | 'C');
    miCopy.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.copy();
      }
    });
    MenuItem miPaste = new MenuItem(editMenu, SWT.NONE);
    miPaste.setText("&Paste\tCtrl-V");
    miPaste.setAccelerator(SWT.CONTROL | 'V');
    miPaste.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.paste();
      }
    });

    MenuItem miDelete = new MenuItem(editMenu, SWT.NONE);
    miDelete.setText("&Delete\tDel");
    miDelete.setAccelerator(SWT.DEL);
    miDelete.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.deleteSelection();
      }
    });
    
    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miUndo = new MenuItem(editMenu, SWT.NONE);
    miUndo.setText("&Undo\tCtrl-Z");
    miUndo.setAccelerator(SWT.CONTROL | 'Z');
    miUndo.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.getDocument().undo();
      }
    });
    MenuItem miRedo = new MenuItem(editMenu, SWT.NONE);
    miRedo.setText("&Redo\tCtrl-Y");
    miRedo.setAccelerator(SWT.CONTROL | 'Y');
    miRedo.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.getDocument().redo();
      }
    });

    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miGroup = new MenuItem(editMenu, SWT.NONE);
    miGroup.setText("Group");
    miGroup.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        groupAction();
      }
    });
    MenuItem miSubgraph = new MenuItem(editMenu, SWT.NONE);
    miSubgraph.setText("Make SubGraph");
    miSubgraph.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        subgraphAction();
      }
    });
    MenuItem miUngroup = new MenuItem(editMenu, SWT.NONE);
    miUngroup.setText("Ungroup");
    miUngroup.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        ungroupAction();
      }
    });

    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miAddLeftPort = new MenuItem(editMenu, SWT.NONE);
    miAddLeftPort.setText("Add Left GeneralNode Port");
    miAddLeftPort.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        addLeftPortAction();
      }
    });
    MenuItem miAddRightPort = new MenuItem(editMenu, SWT.NONE);
    miAddRightPort.setText("Add Right GeneralNode Port");
    miAddRightPort.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        addRightPortAction();
      }
    });

    
    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miDrawStroke = new MenuItem(editMenu, SWT.NONE);
    miDrawStroke.setText("Draw Stroke");
    miDrawStroke.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { insertDrawingStroke(); }
    });
    MenuItem miAutoLayout = new MenuItem(editMenu, SWT.NONE);
    miAutoLayout.setText("AutoLayout");
    miAutoLayout.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        JGoDocument doc = myView.getDocument();
        doc.startTransaction();
        JGoLayeredDigraphAutoLayout l = new JGoLayeredDigraphAutoLayout(doc,
          30, 30,  JGoLayeredDigraphAutoLayout.LD_DIRECTION_RIGHT,
          JGoLayeredDigraphAutoLayout.LD_CYCLEREMOVE_DFS,
          JGoLayeredDigraphAutoLayout.LD_LAYERING_OPTIMALLINKLENGTH,
          JGoLayeredDigraphAutoLayout.LD_INITIALIZE_DFSOUT, 4,
          JGoLayeredDigraphAutoLayout.LD_AGGRESSIVE_FALSE);
        l.performLayout();
        doc.endTransaction("LayeredDigraph Layout");
      }
    });
    MenuItem miTest = new MenuItem(editMenu, SWT.NONE);
    miTest.setText("Stress Test");
    miTest.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        JGoDocument doc = myView.getDocument();
        for (int i = 0; i < 100; i++) {
          for (int j = 0; j < 100; j++) {
            String str = Integer.toString(i) + "," + Integer.toString(j);
            JGoBasicNode obj = new JGoBasicNode(str);
            obj.setToolTipText(str);
            obj.setTopLeft(i*40, j*40);
            switch ((i*100+j) % 6) {
              default:
              case 0: obj.setPen(JGoPen.black); break;
              case 1: obj.setPen(JGoPen.red); break;
              case 2: obj.setPen(JGoPen.gray); break;
              case 3: obj.setPen(JGoPen.black); break;
              case 4: obj.setPen(JGoPen.green); break;
              case 5: obj.setPen(JGoPen.blue); break;
            }
            doc.addObjectAtTail(obj);
          }
        }
      }
    });

    // Create View menu
    MenuItem miView = new MenuItem(mb, SWT.CASCADE);
    miView.setText("&View");
    Menu viewMenu = new Menu(myShell, SWT.DROP_DOWN);
    miView.setMenu(viewMenu);
    
    // Create View menu items
    MenuItem miZoomIn = new MenuItem(viewMenu, SWT.NONE);
    miZoomIn.setText("Zoom In\tCtrl-F6");
    miZoomIn.setAccelerator(SWT.CONTROL | SWT.F6);
    miZoomIn.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.setScale(myView.getScale() * 1.1d);
      }
    });
    MenuItem miZoomOut = new MenuItem(viewMenu, SWT.NONE);
    miZoomOut.setText("Zoom Out\tShift-F6");
    miZoomOut.setAccelerator(SWT.SHIFT | SWT.F6);
    miZoomOut.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.setScale(myView.getScale() / 1.1d);
      }
    });
    MenuItem miZoomNormal = new MenuItem(viewMenu, SWT.NONE);
    miZoomNormal.setText("Zoom Normal\tCtrl-Shift-F6");
    miZoomNormal.setAccelerator(SWT.CONTROL | SWT.SHIFT | SWT.F6);
    miZoomNormal.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myView.setScale(1.0d);
      }
    });

    new MenuItem(viewMenu, SWT.SEPARATOR);

    MenuItem miOverview = new MenuItem(viewMenu, SWT.NONE);
    miOverview.setText("Overview");
    miOverview.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        overviewAction();
      }
    });
    
    MenuItem miProperties = new MenuItem(viewMenu, SWT.NONE);
    miProperties.setText("Properties\tF4");
    miProperties.setAccelerator(SWT.F4);
    miProperties.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        JGoObject obj = myView.getSelection().getPrimarySelection();
        callDialog(obj);  // OK to be null
      }
    });

    myShell.setMenuBar(mb);
  }

  protected void initCanvas() {
    myMainLayer = myDoc.getFirstLayer();
    myForegroundLayer = myDoc.addLayerAfter(myMainLayer);
    myForegroundLayer.setIdentifier("in foreground layer");
    myBackgroundLayer = myDoc.addLayerBefore(myMainLayer);
    myBackgroundLayer.setModifiable(false);
    // by default new user-drawn links go into this layer:
    myDoc.setLinksLayer(myDoc.addLayerBefore(myMainLayer));

    // Set up main view properties
    myView.setGridWidth(20);
    myView.setGridHeight(20);

    stuffAction();
  }

  protected void initPalette() {
    JGoDocument realDoc = myDoc;
    JGoLayer realBack = myBackgroundLayer;
    JGoLayer realMain = myMainLayer;
    JGoLayer realFore = myForegroundLayer;

    JGoDocument paletteDoc = myPalette.getDocument();

    // don't care about undo/redo for the palette,
    // so don't need to call paletteDoc.fireForedate here
    paletteDoc.setSuspendUpdates(true);

    myDoc = paletteDoc;  // temporarily, so objects get created in palette
    myBackgroundLayer = paletteDoc.getDefaultLayer();
    myMainLayer = paletteDoc.getDefaultLayer();
    myForegroundLayer = paletteDoc.getDefaultLayer();
    stuffAction();  // make a bunch of stuff in the palette, into myDoc

    // remove all links between nodes from palette
    ArrayList links = new ArrayList();
    JGoListPosition pos = paletteDoc.getFirstObjectPos();
    while (pos != null) {
      JGoObject obj = paletteDoc.getObjectAtPos(pos);
      pos = paletteDoc.getNextObjectPosAtTop(pos);
      if (obj instanceof JGoLink) {
        links.add(obj);
      }
    }
    for (int i = 0; i < links.size(); i++) {
      paletteDoc.removeObject((JGoObject)links.get(i));
    }

    myDoc = realDoc;  // switch back to normal document
    myBackgroundLayer = realBack;
    myMainLayer = realMain;
    myForegroundLayer = realFore;

    paletteDoc.setSuspendUpdates(false);
    // don't care about undo/redo for the palette,
    // so don't need to call paletteDoc.fireUpdate here

    myPalette.layoutItems();
  }


  static public void main(String args[]) {
    try {
      Display display = new Display();
      Shell shell = new Shell(display);
      shell.setText("Demo1");
      FillLayout layout = new FillLayout();
      shell.setLayout(layout);

      theApp = new Demo1(shell);

      shell.open();
      while (!shell.isDisposed()) {
        if (!display.readAndDispatch())
          display.sleep();
      }
    } catch (Throwable t) {
      System.err.println(t);
      t.printStackTrace();
      System.exit(1);
    }
  }

  static private Demo1 theApp = null;
  static public Demo1 getApp() { return theApp; }

  //==============================================================
  // Process JGoDocumentEvents from the JGoDocument
  //==============================================================
  public void processDocChange(JGoDocumentEvent e)
  {
    switch(e.getHint()) {
      case JGoDocumentEvent.STARTED_TRANSACTION:
        //System.err.println("Started Transaction");
        break;
      case JGoDocumentEvent.FINISHED_TRANSACTION:
        // update the Inspector and any other UI that depends on the state of the document
        Inspector.refresh();
        updateActions();
        //if (e.getPreviousValue() != null)
        //  System.err.println("Finished Transaction " + e.getPreviousValue().toString());
        //else
        //  System.err.println("Finished Transaction");
        break;
      case JGoDocumentEvent.ABORTED_TRANSACTION:
        //System.err.println("Aborted Transaction");
        break;
      case JGoDocumentEvent.STARTING_UNDO:
        //if (e.getObject() instanceof UndoableEdit)
        //  System.err.println("Starting Undo " + ((UndoableEdit)e.getObject()).getUndoPresentationName());
        //else
        //  System.err.println("Starting Undo");
        break;
      case JGoDocumentEvent.FINISHED_UNDO:
        //System.err.println("Finished Undo");
        break;
      case JGoDocumentEvent.STARTING_REDO:
        //if (e.getObject() instanceof UndoableEdit)
        //  System.err.println("Starting Redo " + ((UndoableEdit)e.getObject()).getRedoPresentationName());
        //else
        //  System.err.println("Starting Redo");
        break;
      case JGoDocumentEvent.FINISHED_REDO:
        //System.err.println("Finished Redo");
        break;
      case JGoDocumentEvent.INSERTED:
        if (e.getJGoObject() instanceof JGoLink) {

          // make sure each link that's created has an arrowhead at its front,
          // if the "to" port is a port on a JGoBasicNode
          JGoLink link = (JGoLink)e.getJGoObject();
          JGoPort port = link.getToPort();
          if (port != null) {
            if (port instanceof MultiPortNodePort) {
              link.setArrowHeads(false, true);
            } else if (port.getParent() instanceof JGoBasicNode) {
              // only have an arrowhead at the "to" end
              link.setArrowHeads(false, true);

              // the default values produce a traditional looking arrowhead;
              // the following values produce a diamond instead
              //link.setArrowAngle(Math.PI/3);
              //link.setArrowLength(8);
              //link.setArrowShaftLength(16);
            }

            // if the "from" port is the same as the "to" port, make sure
            // it doesn't have labels at either end, and change its middle label
            if (port == link.getFromPort() && link instanceof JGoLabeledLink) {
              JGoLabeledLink ll = (JGoLabeledLink)link;
              JGoObject mid = ll.getMidLabel();
              ll.setFromLabel(null);
              ll.setToLabel(null);
              if (mid != null && mid instanceof JGoText)
                ((JGoText)mid).setText("loop");
            }
          }

          // if it's connected to a MultiSpotNode, make it orthogonally routed
          // (not implemented as a separate class, so we're using a flag on
          //  the object as a marker)
          if (port != null &&
              port.getParent() != null) {
            if ((port.getParent().getFlags() & flagMultiSpotNode) != 0) {
              link.setOrthogonal(true);
              link.setAvoidsNodes(true);
              link.setJumpsOver(true);
            } else if (port.getParent() instanceof MultiTextNode) {
              link.setOrthogonal(true);
              link.setArrowHeads(false, true);
              link.setArrowShaftLength(0);
            }
          }

          // if the link connects ports on different kinds of objects,
          // highlight it in red
          if (link.getFromPort() != null &&
              port.getClass() != link.getFromPort().getClass()) {
            link.setHighlight(JGoPen.make(JGoPen.SOLID,
                                          link.getPen().getWidth()+4, new RGB(255, 0, 0)));
          }
        }
        break;
    }
  }

  //==============================================================
  // Process JGoViewEvents from the JGoView
  //==============================================================
  public void processViewChange(JGoViewEvent e)
  {
    // if the selection changed, maybe some commands need to
    // be disabled or re-enabled
    switch(e.getHint()) {
      case JGoViewEvent.UPDATE_ALL:
      case JGoViewEvent.SELECTION_GAINED:
      case JGoViewEvent.SELECTION_LOST:
        UpdateControls();
        break;
      case JGoViewEvent.SELECTION_STARTING:
        myUpdatingSelection = true;
        break;
      case JGoViewEvent.SELECTION_FINISHED:
        myUpdatingSelection = false;
        UpdateControls();
        break;
      case JGoViewEvent.DOUBLE_CLICKED: {
        JGoObject obj = e.getJGoObject();
        // get "top-level" object
        //while (obj.getParent() != null)
        //  obj = obj.getParent();
        //callDialog(obj);
        break;
      }
      case JGoViewEvent.BACKGROUND_CLICKED: {
        //MessageBox dlg = new MessageBox(myShell, SWT.ICON_INFORMATION | SWT.OK);
        //dlg.setText("Background Clicked");
        //dlg.setMessage("Clicked at " + e.getPointDocCoords().toString());
        //dlg.open();
      }
      case JGoViewEvent.CLICKED: {
        //if (e.getObject() instanceof Button) {
        //  Button but = (Button)e.getObject();
        //  MessageBox dlg = new MessageBox(myShell, SWT.ICON_INFORMATION | SWT.OK);
        //  dlg.setText("Button Clicked");
        //  dlg.setMessage("Clicked on a Button");
        //  dlg.open();
        //} else if (e.getJGoObject() != null) {
        //  JGoObject obj = e.getJGoObject().getParentNode();
        //  if (obj instanceof JGoBasicNode) {
        //    JGoBasicNode bn = (JGoBasicNode)obj;
        //    startTransaction();
        //    bn.setBrush(JGoBrush.makeStockBrush(getRandomColor()));
        //    endTransaction("changed node color");
        //  }
        //}
      }
    }
  }
  private boolean myUpdatingSelection = false;
  public void UpdateControls() {
    if (myUpdatingSelection) return;
    if (Inspector.getInspector() != null && Inspector.getInspector().isVisible()) {
      Inspector.inspect(myShell.getDisplay(), myShell, myView.getSelection().getPrimarySelection());
    }
    updateActions();
  }

  public void startTransaction()
  {
    myDoc.startTransaction();
  }

  public void endTransaction(String pname)
  {
    myDoc.endTransaction(pname);
  }

  // the default place to put stuff if not dragged there
  Point getDefaultLocation() {
    // to avoid constantly putting things in the same place,
    // keep shifting the default location
    if (myDefaultLocation != null) {
      myDefaultLocation.x += 4;
      myDefaultLocation.y += 4;
    }
    return myDefaultLocation;
  }

  public void setDefaultLocation(Point loc) {
    myDefaultLocation = loc;
  }

  public RGB getRandomColor() {
    int r = (int)(getRandom() * 256);
    int g = (int)(getRandom() * 256);
    int b = (int)(getRandom() * 256);
    return new RGB(r, g, b);
  }

  protected double getRandom() {
    if (isRandom())
      return Math.random();
    else
      return 0.57;
  }

  public void setRandom(boolean isRandom)  {myRandom = isRandom;}

  public boolean isRandom() {return myRandom;}

  public void setMainLayer(JGoLayer layer) {
    myMainLayer = layer;
  }
  public void setForegroundLayer(JGoLayer layer) {
    myForegroundLayer = layer;
  }


  //==============================================================
  // Create various kinds of objects
  //==============================================================

  public void stuffAction()
  {
    startTransaction();
    Point oldloc = getDefaultLocation();
    setDefaultLocation(null); // position everything at absolute default locations
    rectangleAction();
    ellipseAction();
    multiPortNodeAction();
    simpleNodeAction();
    JGoPort p1 = iconicNodeAction();
    JGoPort p2 = draggableLabelIconicNodeAction();
    basicNodeAction();
    roundedRectangleAction();
    centerLabelBasicNodeAction();
    JGoPort p3 = fixedSizeCenterLabelBasicNodeAction();
    JGoPort p5 = rectBasicNodeAction();
    selfLoopBasicNodeAction();
    multiSpotNodeAction();
    generalNodeAction();
    textNodeAction();
    JGoPort p4 = fixedSizeTextNodeAction();

    JGoLink link1 = new JGoLink();
    link1.setArrowHeads(false, true);
    link1.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorMagenta));
    link1.setCubic(true);
    link1.setFromPort(p1);
    link1.setToPort(p2);
    myMainLayer.addObjectAtTail(link1);

    JGoLink link2 = new JGoLink(p3, p4);
    myMainLayer.addObjectAtTail(link2);

    JGoLabeledLink link3 = new JGoLabeledLink(p5, p3);
    JGoLinkLabel lab3 = new JGoLinkLabel("mid");
    link3.setArrowHeads(false, true);
    link3.setArrowShaftLength(0);
    link3.setMidLabel(lab3);
    link3.setBrush(null);
    myMainLayer.addObjectAtTail(link3);

    JGoText t = new JGoText();
    t.setLocation(new Point(70, 10));
    t.setText("Hello!\nWWW");
    t.setTextColor(new RGB(0, 0, 200));
    t.setBkColor(new RGB(200, 200, 255));
    t.setBold(true);
    t.setItalic(true);
    t.setUnderline(true);
    t.setStrikeThrough(true);
    t.setMultiline(true);
    t.setWrapping(true);
    t.setEditable(true);
    t.setEditOnSingleClick(true);
    myDoc.addObjectAtTail(t);

    JGoImage img = new JGoImage(new Rectangle(25,375,50,50));
    img.loadImage(Demo1.class.getResource("star.gif"), false);
    myDoc.addObjectAtTail(img);

    TestIconicNode tin = new TestIconicNode("test");
    tin.getImage().loadImage(Demo1.class.getResource("star.gif"), false);
    tin.setTopLeft(10, 450);
    myDoc.addObjectAtTail(tin);

    TestSubGraph sg = new TestSubGraph("test subgraph");

    JGoImage sgimg = new JGoImage();
    sgimg.loadImage(Demo1.class.getResource("star.gif"), true);
    sgimg.setSize(40, 40);
    sgimg.setSelectable(false);
    sgimg.setDragsNode(true);
    sgimg.setVisible(false);
    sg.setCollapsedObject(sgimg);

    JGoBasicNode sgbn1 = new JGoBasicNode("bn1");
    sgbn1.setLocation(100, 500);
    sg.addObjectAtTail(sgbn1);
    JGoBasicNode sgbn2 = new JGoBasicNode("bn2");
    sgbn2.setLocation(200, 500);
    sg.addObjectAtTail(sgbn2);
    JGoLabeledLink sglink1 = new JGoLabeledLink(sgbn1.getPort(), sgbn2.getPort());
    JGoLinkLabel lllabel = new JGoLinkLabel();
    lllabel.setText("middle");
    sglink1.setMidLabel(lllabel);
    sglink1.setArrowHeads(false, true);
    sg.addObjectAtTail(sglink1);
    myDoc.addObjectAtTail(sg);

    TestSubGraph2 sg2 = new TestSubGraph2("test subgraph2");

    JGoImage sgimg2 = new JGoImage();
    sgimg2.loadImage(Demo1.class.getResource("star.gif"), true);
    sgimg2.setSize(40, 40);
    sgimg2.setSelectable(false);
    sgimg2.setDragsNode(true);
    sgimg2.setVisible(false);
    sg2.setCollapsedObject(sgimg2);

    JGoBasicNode sg2bn1 = new JGoBasicNode("bn1");
    sg2bn1.setLocation(300, 500);
    sg2.addObjectAtTail(sg2bn1);
    JGoBasicNode sg2bn2 = new JGoBasicNode("bn2");
    sg2bn2.setLocation(400, 500);
    sg2.addObjectAtTail(sg2bn2);
    JGoLink sg2link1 = new JGoLink(sg2bn1.getPort(), sg2bn2.getPort());
    sg2link1.setArrowHeads(false, true);
    sg2.addObjectAtTail(sg2link1);
    myDoc.addObjectAtTail(sg2);

    roundTextNodeAction();
    multiTextNodeAction();
    strokeAction();
    polygonAction();
    diamondAction();
    textAction();
    threeDRectAction();
    commentAction();
    listAreaAction();
    recordNodeAction();
    setDefaultLocation(oldloc);

    insertGraphOfGraphs();

    endTransaction("inserted stuff");
  }

  // bring up the Inspector on the primary selection
  void inspectAction() {
    Inspector.inspect(myShell.getDisplay(), myShell, myView.getSelection().getPrimarySelection());
    Composite inspector = Inspector.getInspector();
    if (inspector != null && !inspector.isVisible())
      inspector.setVisible(true);
  }

  // bring up a JGoOverview window
  void overviewAction() {
    Shell shell = new Shell(myShell, SWT.DIALOG_TRIM | SWT.RESIZE);
    shell.setText("Overview");
    shell.setSize(200, 200);
    FillLayout layout = new FillLayout();
    shell.setLayout(layout);
    JGoOverview ov = new JGoOverview(shell, SWT.V_SCROLL | SWT.H_SCROLL);
    ov.setObserved(myView);
    shell.open();
  }

  // this calls the appropriate dialog box for the type of object that obj is.
  // By default, we just bring up the Inspector.
  void callDialog(JGoObject obj)
  {
    startTransaction();
    myView.getSelection().clearSelectionHandles(obj);
    String title = null;
    Inspector.inspect(myShell.getDisplay(), myShell, obj);
    Composite inspector = Inspector.getInspector();
    if (inspector != null && !inspector.isVisible())
      inspector.setVisible(true);
    myDoc.update();
    myView.getSelection().restoreSelectionHandles(obj);
    endTransaction(title);
  }

  // this enables/disables all known actions
  static public void updateActions()
  {
  }

  static public JGoDocument loadObjects(InputStream ins)
    throws IOException, ClassNotFoundException
  {
    ObjectInputStream istream = new ObjectInputStream(ins);
    Object newObj = istream.readObject();
    if (newObj instanceof JGoDocument) {
      JGoDocument doc = (JGoDocument)newObj;
      return doc;
    } else {
      return null;
    }
  }

  public void storeObjects(OutputStream outs)
    throws IOException
  {
    ObjectOutputStream ostream = new ObjectOutputStream(outs);
    ostream.writeObject(myView.getDocument());
    ostream.flush();
  }

  public void rectangleAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(180, 250);
    JGoRectangle obj = new JGoRectangle(loc, new Dimension(75,75));
    obj.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorRed));
    myMainLayer.addObjectAtTail(obj);
    endTransaction("inserted rectangle");
  }

  void roundedRectangleAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(180, 150);
    JGoRoundRect roundRect = new JGoRoundRect(loc, new Dimension(80,80), new Dimension(15,30));
    roundRect.setPen(JGoPen.make(JGoPen.DASHDOTDOT, 3, JGoBrush.ColorDarkGray));
    // brush gets determined by Demo1RoundRect
    myMainLayer.addObjectAtTail(roundRect);
    endTransaction("inserted rounded rectangle");
  }

  public void ellipseAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(100, 250);
    JGoEllipse ellipse = new JGoEllipse(loc, new Dimension(50,75));
    ellipse.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorGreen));
    myMainLayer.addObjectAtTail(ellipse);
    endTransaction("inserted ellipse");
  }

  void multiPortNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(10, 170);
    MultiPortNode snode = new MultiPortNode();
    JGoImage nodeicon = new JGoImage(new Rectangle(0,0,40,40));
    nodeicon.loadImage(Demo1.class.getResource("doc.gif"), true);
    snode.initialize(loc, nodeicon, "multiport node");

    int numports = (int)(getRandom()*5)+1;
    for (int i = 0; i < numports; i++) {
      Point offset = new Point(8*((int)(getRandom()*6)-1), 8*((int)(getRandom()*6)-1));
      int spot = JGoObject.Center;
      if (offset.x > offset.y) {
        if (offset.x < 40 - offset.y)
          spot = JGoObject.TopMiddle;
        else
          spot = JGoObject.SideRight;
      } else {
        if (offset.x > 40 - offset.y)
          spot = JGoObject.BottomMiddle;
        else
          spot = JGoObject.SideLeft;
      }
      MultiPortNodePort p = new MultiPortNodePort(true, true, spot,
        offset, new Dimension(8, 8), nodeicon, snode);
    }

    myMainLayer.addObjectAtTail(snode);
    endTransaction("inserted MultiPortNode");
  }

  public void simpleNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(100, 70);
    SimpleNode snode = new SimpleNode();
    JGoImage nodeicon = new JGoImage(new Rectangle(0,0,50,50));
    nodeicon.loadImage(Demo1.class.getResource("doc.gif"), true);
    snode.initialize(loc, new Dimension(50,50), nodeicon, "a simple node", true, true);
    myMainLayer.addObjectAtTail(snode);
    endTransaction("inserted SimpleNode");
  }

  public JGoPort iconicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(340, 110);
    JGoIconicNode snode = new JGoIconicNode("an iconic node");
    snode.setToolTipText("an iconic node's tooltip");
    // modify the iconic node's standard image to display a GIF
    snode.getImage().setSize(50, 50);
    snode.getImage().loadImage(Demo1.class.getResource("doc.gif"), true);
    snode.setLocation(loc);
    myMainLayer.addObjectAtTail(snode);
    endTransaction("inserted IconicNode");
    return snode.getPort();
  }

  JGoPort draggableLabelIconicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(440, 110);
    JGoIconicNode snode = new JGoIconicNode("draggable label");
    snode.setToolTipText("a tooltip");
    // replace the iconic node's standard image with a new one displaying a GIF
    JGoImage nodeicon = new JGoImage(new Rectangle(0,0,50,50));
    nodeicon.loadImage(Demo1.class.getResource("doc.gif"), true);
    snode.setIcon(nodeicon);
    snode.setLocation(loc);
    snode.setDraggableLabel(true);
    myMainLayer.addObjectAtTail(snode);
    endTransaction("inserted draggable Label IconicNode");
    return snode.getPort();
  }


  // let each newly created JGoBasicNode show an integer label
  int basicNodeCounter = 1;

  public JGoPort basicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(40, 140);
    JGoBasicNode bnode = new JGoBasicNode(Integer.toString(basicNodeCounter));
    bnode.setToolTipText("a basic node's tooltip");
    if (basicNodeCounter%18 >= 9) {
      JGoRectangle rect = new JGoRectangle();
      rect.setSize(20, 20);
      bnode.setDrawable(rect);
    }
    bnode.setLocation(loc);
    bnode.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorOrange));
    bnode.setLabelSpot(basicNodeCounter%9);
    myMainLayer.addObjectAtTail(bnode);
    basicNodeCounter++;
    endTransaction("inserted BasicNode");
    return bnode.getPort();
  }

  JGoPort centerLabelBasicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(270, 30);
    JGoBasicNode tnode = new JGoBasicNode("basic node\ncenter label");
    tnode.setToolTipText("a centered basic node's tooltip");
    tnode.setInsets(new Insets(10, 18, 10, 18));
    tnode.getLabel().setMultiline(true);
    tnode.setLabelSpot(JGoObject.Center);
    tnode.setBrush(JGoBrush.makeStockBrush(new RGB(0x90, 0xEE, 0x90)));
    tnode.setPen(JGoPen.makeStockPen(new RGB(0xFF, 0x14, 0x93)));
    tnode.setLocation(loc);
    myMainLayer.addObjectAtTail(tnode);
    endTransaction("inserted CenterLabel BasicNode");
    return tnode.getPort();
  }

  JGoPort fixedSizeCenterLabelBasicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(340, 350);
    JGoBasicNode tnode = new JGoBasicNode("resizable basic node with AutoResize false");
    tnode.setToolTipText("a resizable basic node's tooltip");
    tnode.setInsets(new Insets(10, 18, 10, 18));
    tnode.getLabel().setMultiline(true);
    tnode.setLabelSpot(JGoObject.Center);
    tnode.setBrush(JGoBrush.makeStockBrush(new RGB(0x7F, 0xFF, 0xD4)));
    tnode.setLocation(loc);
    tnode.setResizable(true);
    tnode.setAutoResize(false);
    tnode.setSize(200, 50);
    myMainLayer.addObjectAtTail(tnode);
    endTransaction("inserted FixedSize CenterLabel BasicNode");
    return tnode.getPort();
  }

  JGoPort rectBasicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(90, 350);
    JGoBasicNode tnode = new JGoBasicNode("rectangular basic node");
    tnode.setToolTipText("a rectangular basic node's tooltip");
    tnode.setLabelSpot(JGoObject.Center);
    tnode.setDrawable(new JGoRectangle());
    tnode.setBrush(JGoBrush.makeStockBrush(new RGB(0xC0, 0, 0xC0)));
    tnode.setLocation(loc);
    myMainLayer.addObjectAtTail(tnode);
    endTransaction("inserted Rectangular BasicNode");
    return tnode.getPort();
  }

  public JGoPort selfLoopBasicNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(270, 100);
    JGoBasicNode tnode = new JGoBasicNode("self loop");
    tnode.setToolTipText("self loop tooltip");
    tnode.setInsets(new Insets(10, 18, 10, 18));
    tnode.getLabel().setMultiline(true);
    tnode.setLabelSpot(JGoObject.Center);
    tnode.setLocation(loc);
    tnode.setBrush(JGoBrush.makeStockBrush(new RGB(0x80, 0x00, 0x80)));
    tnode.getLabel().setBold(true);
    tnode.getLabel().setTextColor(JGoBrush.ColorWhite);
    tnode.getPort().setValidSelfNode(true);
    myMainLayer.addObjectAtTail(tnode);
    JGoLink l = new JGoLink(tnode.getPort(), tnode.getPort());
    l.setArrowShaftLength(0);
    l.setArrowLength(6);
    l.setArrowWidth(8);
    l.setPen(JGoPen.makeStockPen(new RGB(0x80, 0x00, 0x80)));
    myMainLayer.addObjectAtTail(l);
    l.calculateStroke();
    endTransaction("inserted SelfLoop BasicNode");
    return tnode.getPort();
  }

  JGoPort textNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(120, 20);
    JGoTextNode tnode = new JGoTextNode("text node");
    tnode.setToolTipText("a text node's tooltip");
    tnode.setTopLeft(loc);
    tnode.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorPink));
    myMainLayer.addObjectAtTail(tnode);
    endTransaction("inserted TextNode");
    return tnode.getBottomPort();
  }

  JGoPort fixedSizeTextNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(290, 230);
    JGoTextNode tnode = new JGoTextNode("resizable text node with AutoResize false");
    tnode.setToolTipText("a resizable text node's tooltip");
    tnode.setResizable(true);
    tnode.setAutoResize(false);
    tnode.setTopLeft(loc);
    tnode.setSize(100, 100);
    tnode.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorYellow));
    tnode.getLabel().setEditable(true);
    tnode.getLabel().setAlignment(JGoText.ALIGN_CENTER);
    myMainLayer.addObjectAtTail(tnode);
    endTransaction("inserted FixedSize TextNode");
    return tnode.getLeftPort();
  }

  JGoPort roundTextNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(320, 10);
    JGoTextNode rnode = new JGoTextNode("rounded text node\ndisplaying three lines\nwith wider insets");
    rnode.setToolTipText("a rounded text node's tooltip");
    rnode.setTopLeft(new Point(loc.x+40, loc.y));
    rnode.getLabel().setEditable(true);
    JGoRoundRect rrect = new JGoRoundRect(new Dimension(10, 10));
    rrect.setBrush(JGoBrush.white);
    rnode.setBackground(rrect);
    rnode.setInsets(new Insets(4, 10, 4, 10));
    myMainLayer.addObjectAtTail(rnode);
    endTransaction("inserted Round TextNode");
    return rnode.getTopPort();
  }

  void multiTextNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(370, 160);

    MultiTextNode tnode = new MultiTextNode();
    tnode.initialize();
    tnode.setTopLeft(loc);
    JGoText t = (JGoText)tnode.addString("MultiTextNode");
    t.setAlignment(JGoText.ALIGN_CENTER);
    t = (JGoText)tnode.addString("second line");
    t.setAlignment(JGoText.ALIGN_CENTER);
    t.setBold(true);
    t = (JGoText)tnode.addString("third line");
    t.setAlignment(JGoText.ALIGN_CENTER);
    tnode.setLinePen(JGoPen.darkGray);
    //JGoRoundRect rr = new JGoRoundRect();
    //rr.setSelectable(false);
    //rr.setArcDimension(new Dimension(20, 20));
    //tnode.setBackground(rr);
    //tnode.setInsets(new Insets(5, 5, 5, 5));  // make room for corners
    tnode.setInsets(new Insets(0, 2, 0, 2));  // add space on sides
    tnode.setItemWidth(100);
    myForegroundLayer.addObjectAtTail(tnode);

    endTransaction("inserted MultiTextNode");
  }

  public void recordNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(500, 100);

    RecordNode right = makeRecordNode(true, myRNorg);
    right.setTopLeft(loc.x + 200, loc.y);
    right.setHeight(150);
    right.setLinePen(JGoPen.gray);
    right.setSpacing(3);

    RecordNode left = makeRecordNode(false, myRNorg);
    left.setTopLeft(loc);
    left.setHeight(150);

    myRNorg = !myRNorg;

    JGoPort left5 = left.getRightPort(5);
    JGoPort left6 = left.getRightPort(6);
    JGoPort left7 = left.getRightPort(7);
    JGoPort left8 = left.getRightPort(8);
    JGoPort right5 = right.getLeftPort(5);
    JGoPort right10 = right.getLeftPort(10);
    JGoPort right0 = right.getLeftPort(0);
    JGoPort right19 = right.getLeftPort(19);
    if (left5 != null && right5 != null)
      myForegroundLayer.addObjectAtTail(new JGoLink(left5, right5));
    JGoImage star = new JGoImage();
    star.loadImage(Demo1.class.getResource("star.gif"), true);
    if (right5 != null) {
      right5.setSize(7, 7);
      right5.setStyle(JGoPort.StyleObject);
      right5.setPortObject(star);
    }
    if (left6 != null && right10 != null)
      myForegroundLayer.addObjectAtTail(new JGoLink(left6, right10));
    if (right10 != null) {
      right10.setSize(7, 7);
      right10.setPortObject(star);
      right10.setStyle(JGoPort.StyleObject);
    }
    if (left7 != null && right0 != null)
      myForegroundLayer.addObjectAtTail(new JGoLink(left7, right0));
    if (right0 != null) {
      right0.setSize(7, 7);
      right0.setPortObject(star);
      right0.setStyle(JGoPort.StyleObject);
    }
    if (left8 != null && right19 != null)
      myForegroundLayer.addObjectAtTail(new JGoLink(left8, right19));
    if (right19 != null) {
      right19.setSize(7, 7);
      right19.setPortObject(star);
      right19.setStyle(JGoPort.StyleObject);
    }
    if (left5 != null && right0 != null)
      myForegroundLayer.addObjectAtTail(new JGoLink(left5, right0));
    JGoPort left13 = left.getRightPort(13);
    if (left13 != null) {
      left13.setSize(9, 9);
      left13.setStyle(JGoPort.StyleDiamond);
      left13.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorMagenta));
    }

    endTransaction("inserted RecordNode");
  }

  static boolean myRNorg = true;

  RecordNode makeRecordNode(boolean onRight, boolean vertical) {
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(20, 250);
    RecordNode tnode = new RecordNode();
    tnode.initialize();
    tnode.setScrollBarOnRight(onRight);
    tnode.setVertical(vertical);
    //tnode.setAlignment(ListArea.ALIGN_RIGHT);  // icons on left, text on right

    JGoText title = new JGoText("a Record");
    title.setAlignment(JGoText.ALIGN_CENTER);
    title.setBkColor(JGoBrush.ColorBlue);
    title.setTextColor(JGoBrush.ColorWhite);
    title.setBold(true);
    title.setAutoResize(false);
    title.setClipping(true);

    if (!tnode.isVertical() &&
      onRight)
      tnode.setFooter(title);
    else
      tnode.setHeader(title);

    for (int i = 0; i < 20; i++) {
      JGoObject item = makeListItem(i);

      JGoImage icon = new JGoImage();
      icon.loadImage(Demo1.class.getResource("doc.gif"), true);
      icon.setSelectable(false);
      if (tnode.isVertical())
        icon.setSize(10, 12);
      else
        icon.setSize(30, 40);

      JGoPort leftport = null;
      JGoPort rightport = null;
      if (tnode.isScrollBarOnRight()) {
        leftport = new JGoPort();
        leftport.setSize(5, 5);
        leftport.setFromSpot(JGoObject.LeftCenter);
        leftport.setToSpot(JGoObject.LeftCenter);
      } else {
        rightport = new JGoPort();
        rightport.setSize(5, 5);
        rightport.setFromSpot(JGoObject.RightCenter);
        rightport.setToSpot(JGoObject.RightCenter);
      }

      tnode.addItem(item, leftport, rightport, icon);
    }

    myForegroundLayer.addObjectAtTail(tnode);
    return tnode;
  }

  public ListArea listAreaAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(420, 350);
    ListArea tnode = new ListArea();
    tnode.initialize();
    // have fun trying different orientations and scroll bar arrangements
    switch (myLAorg) {
      default:
      case 0:  tnode.setVertical(true);  tnode.setScrollBarOnRight(true);
        myLAorg++; break;
      case 1:  tnode.setVertical(true);  tnode.setScrollBarOnRight(false);
        myLAorg++; break;
      case 2:  tnode.setVertical(false); tnode.setScrollBarOnRight(true);
        myLAorg++; break;
      case 3:  tnode.setVertical(false); tnode.setScrollBarOnRight(false);
        myLAorg = 0; break;
    }
    for (int i = 0; i < 10; i++) {
      tnode.addItem(makeListItem(i), null, null, null);
    }
    myMainLayer.addObjectAtTail(tnode);
    tnode.setTopLeft(loc.x, loc.y);
    // start it not with the first object (0)
    tnode.setFirstVisibleIndex(3);
    tnode.setAlignment(ListArea.ALIGN_LEFT);
    endTransaction("inserted ListArea");
    return tnode;
  }

  static int myLAorg = 0;

  JGoObject makeListItem(int i) {
    // just for fun, have a differently sized (and looking) item
    if (i == 7) {
      JGoPolygon gon = new JGoPolygon();
      gon.setBrush(JGoBrush.red);
      gon.setPen(JGoPen.make(JGoPen.SOLID, 3, JGoBrush.ColorWhite));
      gon.addPoint(new Point(10,  0));
      gon.addPoint(new Point(20,  0));
      gon.addPoint(new Point(30, 10));
      gon.addPoint(new Point(30, 20));
      gon.addPoint(new Point(20, 30));
      gon.addPoint(new Point(10, 30));
      gon.addPoint(new Point( 0, 20));
      gon.addPoint(new Point( 0, 10));
      return gon;
    } else if (i == 13) {
      JGoText t = new JGoText("start a link");
      t.setDraggable(false);
      t.setTransparent(true);
      t.setSelectBackground(true);
      t.setTextColor(JGoBrush.ColorBlue);
      t.setBkColor(myView.getSecondarySelectionColor());
      t.setItalic(true);
      t.setFontSize(16);
      return t;
    } else if (i == 14) {
      JGoText t = new JGoText("not selectable");
      t.setSelectable(false);
      t.setTransparent(true);
      t.setSelectBackground(true);
      t.setTextColor(JGoBrush.ColorRed);
      t.setBkColor(myView.getSecondarySelectionColor());
      t.setItalic(true);
      return t;
    } else {
      String m = "Item " + i;
      JGoText t = new JGoText(m);
      t.setTransparent(true);
      t.setSelectBackground(true);
      t.setBkColor(myView.getSecondarySelectionColor());
      t.setFaceName("Helvetica");
      // for fun, if it's a prime number, make it bold
      if (i == 2 || i == 3 || i == 5 || i == 7 ||
          i == 11 || i == 13 || i == 17 || i == 19)
        t.setBold(true);
      return t;
    }
  }

  // use a flag on JGoObject to recognize a particular kind of area
  // (could have implemented our own subclass of JGoArea, but didn't
  //  want to bother this time)
  public static final int flagMultiSpotNode = 0x10000;

  void multiSpotNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(280, 175);
    JGoArea node = new JGoNode();
    node.setFlags(node.getFlags() | flagMultiSpotNode);
    JGoRectangle rect = new JGoRectangle(loc, new Dimension(50,50));
    rect.setSelectable(false);
    rect.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorGray));
    node.addObjectAtHead(rect);
    addMultiSpotPort(node, rect, JGoObject.Center);
    addMultiSpotPort(node, rect, JGoObject.TopLeft);
    addMultiSpotPort(node, rect, JGoObject.TopCenter);
    addMultiSpotPort(node, rect, JGoObject.TopRight);
    addMultiSpotPort(node, rect, JGoObject.RightCenter);
    addMultiSpotPort(node, rect, JGoObject.BottomRight);
    addMultiSpotPort(node, rect, JGoObject.BottomCenter);
    addMultiSpotPort(node, rect, JGoObject.BottomLeft);
    addMultiSpotPort(node, rect, JGoObject.LeftCenter);
    myMainLayer.addObjectAtTail(node);
    endTransaction("inserted MultiSpot Node");
  }

  void addMultiSpotPort(JGoArea area, JGoObject rect, int spot) {
    JGoPort port = new JGoPort();
    port.setSize(6,6);
    port.setStyle(JGoPort.StyleDiamond);
    port.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorMagenta));
    port.setSpotLocation(spot, rect, spot);
    port.setFromSpot(spot);
    port.setToSpot(spot);
    area.addObjectAtTail(port);
  }

  // create a GeneralNode with a random number of input and output ports
  void generalNodeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(10, 230);
    GeneralNode node = new GeneralNode();
    JGoImage nodeicon = new JGoImage(new Rectangle(0,0,50,50));
    nodeicon.loadImage(Demo1.class.getResource("doc.gif"), true);
    node.initialize(loc, new Dimension(50, 50), nodeicon, "top", "general node",
                    (int)(getRandom()*5), (int)(getRandom()*5));
    myMainLayer.addObjectAtTail(node);
    endTransaction("inserted GeneralNode");
  }

  void addLeftPortAction() {
    JGoObject obj = myView.getSelection().getPrimarySelection();
    if (obj != null && obj.getParentNode() instanceof GeneralNode) {
      startTransaction();
      GeneralNode gn = (GeneralNode)obj.getParentNode();
      String pname = "in" + Integer.toString(gn.getNumLeftPorts());
      GeneralNodePort p = new GeneralNodePort(true, pname, gn);
      GeneralNodePortLabel l = new GeneralNodePortLabel(pname, p);
      gn.addLeftPort(p);
      endTransaction("Added LeftPort");
    }
  }

  void addRightPortAction() {
    JGoObject obj = myView.getSelection().getPrimarySelection();
    if (obj != null && obj.getParentNode() instanceof GeneralNode) {
      startTransaction();
      GeneralNode gn = (GeneralNode)obj.getParentNode();
      String pname = "out" + Integer.toString(gn.getNumRightPorts());
      GeneralNodePort p = new GeneralNodePort(false, pname, gn);
      GeneralNodePortLabel l = new GeneralNodePortLabel(pname, p);
      gn.addRightPort(p);
      endTransaction("Added RightPort");
    }
  }

  void strokeAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(10, 10);
    JGoStroke stroke = new JGoStroke();
    stroke.addPoint(new Point(loc.x+0, loc.y+0));
    stroke.addPoint(new Point(loc.x+68, loc.y+46));
    stroke.addPoint(new Point(loc.x+68, loc.y+90));
    stroke.addPoint(new Point(loc.x+53, loc.y+57));
    stroke.addPoint(new Point(loc.x+85, loc.y+34));
    myMainLayer.addObjectAtTail(stroke);
    endTransaction("inserted Stroke");
  }

  void diamondAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(175, 100);
    Diamond dia3 = new Diamond(loc, new Dimension(70, 40));
    dia3.setPen(JGoPen.make(JGoPen.SOLID, 3, JGoBrush.ColorMagenta));
    myMainLayer.addObjectAtTail(dia3);
    endTransaction("inserted Diamond");
  }

  void polygonAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(10, 80);
    JGoPolygon gon = new JGoPolygon();
    gon.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorRed));
    gon.setPen(JGoPen.make(JGoPen.SOLID, 3, JGoBrush.ColorLightGray));
    gon.addPoint(new Point(loc.x+10, loc.y+ 0));
    gon.addPoint(new Point(loc.x+20, loc.y+ 0));
    gon.addPoint(new Point(loc.x+30, loc.y+10));
    gon.addPoint(new Point(loc.x+30, loc.y+20));
    gon.addPoint(new Point(loc.x+20, loc.y+30));
    gon.addPoint(new Point(loc.x+10, loc.y+30));
    gon.addPoint(new Point(loc.x+ 0, loc.y+20));
    gon.addPoint(new Point(loc.x+ 0, loc.y+10));
    myMainLayer.addObjectAtTail(gon);
    endTransaction("inserted Polygon");
  }

  void textAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(150, 50);
    JGoText text = new JGoText(loc,
      12,
      "Sample Text",
      "Serif",
      true,
      true,
      true,
      JGoText.ALIGN_CENTER,
      false,
      true);
    text.setResizable(true);
    text.setEditOnSingleClick(true);
    myMainLayer.addObjectAtTail(text);
    endTransaction("inserted Text");
  }

  void threeDRectAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(100, 150);
    JGo3DRect obj = new JGo3DRect(loc, new Dimension(55,25));
    obj.setBrush(JGoBrush.makeStockBrush(JGoBrush.ColorCyan));
    myMainLayer.addObjectAtTail(obj);
    endTransaction("inserted 3DRect");
  }

  void commentAction() {
    startTransaction();
    Point loc = getDefaultLocation();
    if (loc == null) loc = new Point(100, 200);
    Comment obj = new Comment("This is a\nmultiline comment.");
    obj.setTopLeft(loc.x, loc.y);
    myMainLayer.addObjectAtTail(obj);
    endTransaction("inserted Comment");
  }

  void insert10000Action() {
    startTransaction();
    JGoPen red = JGoPen.makeStockPen(JGoBrush.ColorRed);
    JGoPen green = JGoPen.makeStockPen(JGoBrush.ColorGreen);
    JGoPen blue = JGoPen.makeStockPen(JGoBrush.ColorBlue);
    for (int i = 0; i < 100; i++) {
      for (int j = 0; j < 100; j++) {
        // JGoDrawable obj = new JGoRectangle();
        // obj.setResizable(false);
        // obj.setTopLeft(i*40, j*40);
        // obj.setSize(20, 20);
        JGoBasicNode obj = new JGoBasicNode(Integer.toString(i) + "," + Integer.toString(j));
        obj.setLocation(new Point(i*40, j*40));
        switch ((i*100+j) % 6) {
          default:
          case 0: obj.setPen(JGoPen.black); break;
          case 1: obj.setPen(red); break;
          case 2: obj.setPen(JGoPen.gray); break;
          case 3: obj.setPen(JGoPen.black); break;
          case 4: obj.setPen(green); break;
          case 5: obj.setPen(blue); break;
        }
        myMainLayer.addObjectAtTail(obj);
      }
    }
    endTransaction("inserted 10000 BasicNodes");
  }

  void insertDrawingStroke() {
    MessageBox dlg = new MessageBox(myShell, SWT.ICON_INFORMATION | SWT.OK);
    dlg.setText("Drawing Stroke Mode");
    dlg.setMessage(
      "Starting a mode to allow you to draw a stroke by specifying its points.\n" +
      "This mode stops when you type ENTER to accept the stroke or ESCAPE to cancel the new link.");
    dlg.open();
    myView.startDrawingStroke();
  }

  public void insertGraphOfGraphs() {
    startTransaction();
    JGoSubGraph gr1 = makeSubGraph("one", "1");
    gr1.setTopLeft(300, 100);
    JGoSubGraph gr2 = makeSubGraph("two", "2");
    gr2.setCollapsible(false);
    gr2.setTopLeft(100, 300);
    gr2.setBackgroundColor(new RGB(200, 200, 63));
    gr2.setBorderPen(JGoPen.make(JGoPen.DASHDOT, 2, JGoBrush.ColorGray));
    JGoSubGraph gr3 = makeSubGraph("three", "3");
    gr3.setTopLeft(500, 300);
    gr3.setBackgroundColor(new RGB(255, 200, 63));
    gr3.setBorderPen(JGoPen.makeStockPen(JGoBrush.ColorMagenta));
    JGoLabeledLink l2 = new JGoLabeledLink(findBasicNode(gr1, "1b").getPort(),
      findBasicNode(gr2, "2a").getPort());
    JGoLinkLabel l2l = new JGoLinkLabel("one to two");
    l2.setMidLabel(l2l);
    JGoLabeledLink l3 = new JGoLabeledLink(findBasicNode(gr1, "1c").getPort(),
      findBasicNode(gr3, "3a").getPort());
    JGoLinkLabel l3l = new JGoLinkLabel("one to three");
    l3.setMidLabel(l3l);
    myMainLayer.addObjectAtTail(gr1);
    myMainLayer.addObjectAtTail(gr2);
    myMainLayer.addObjectAtTail(gr3);
    myMainLayer.addObjectAtTail(l2);
    myMainLayer.addObjectAtTail(l3);
    endTransaction("inserted SubGraph Of SubGraphs");
    startTransaction();
    JGoSubGraph gr4 = makeSubGraph("four", "4");
    gr4.setTopLeft(550, 350);
    gr3.addObjectAtTail(gr4);
    endTransaction("add GR4");
  }

  JGoSubGraph makeSubGraph(String labtext, String prefix) {
    JGoBasicNode a = new JGoBasicNode(prefix + "a");
    a.setLocation(new Point(30, 0));
    JGoBasicNode b = new JGoBasicNode(prefix + "b");
    b.setLocation(new Point(0, 60));
    JGoBasicNode c = new JGoBasicNode(prefix + "c");
    c.setLocation(new Point(60, 60));
    JGoLink ab = new JGoLink(a.getPort(), b.getPort());
    JGoLink ac = new JGoLink(a.getPort(), c.getPort());

    JGoSubGraph gr = new JGoSubGraph(labtext);
    gr.addObjectAtTail(a);
    gr.addObjectAtTail(b);
    gr.addObjectAtTail(c);
    gr.addObjectAtTail(ab);
    gr.addObjectAtTail(ac);
    return gr;
  }

  JGoBasicNode findBasicNode(JGoObjectSimpleCollection sg, String lab) {
    JGoListPosition pos = sg.getFirstObjectPos();
    while (pos != null) {
      JGoObject obj = sg.getObjectAtPos(pos);
      pos = sg.getNextObjectPosAtTop(pos);
      if (obj instanceof JGoBasicNode) {
        JGoBasicNode n = (JGoBasicNode)obj;
        if (n.getLabel().getText().equals(lab)) {
          return n;
        }
      }
    }
    return null;
  }

  // make a JGoNode out of the current selection
  void groupAction() {
    startTransaction();
    JGoSelection selection = myView.getSelection();
    JGoObject primary = selection.getPrimarySelection();
    JGoArea area = new JGoNode();
    // now add the area to the document
    primary.getLayer().addObjectAtTail(area);
    // add the selected objects to the area
    ArrayList arr = area.addCollection(selection, false, myMainLayer);
    // update the selection too, for the user's convenience
    selection.selectObject(area);
    // make each object not Selectable
    for (int i = 0; i < arr.size(); i++) {
      JGoObject obj = (JGoObject)arr.get(i);
      obj.setSelectable(false);
    }
    endTransaction("make area");
  }

  // make a JGoSubGraph out of the current selection
  void subgraphAction() {
    startTransaction();
    JGoSelection selection = myView.getSelection();
    JGoObject primary = selection.getPrimarySelection();
    JGoSubGraph sg = new JGoSubGraph();
    sg.initialize("subgraph!");
    // now add the area to the document
    primary.getLayer().addObjectAtTail(sg);
    // add the selected objects to the area
    sg.addCollection(selection, true, myMainLayer);
    // update the selection too, for the user's convenience
    selection.selectObject(sg);
    endTransaction("make subgraph");
  }

  // this action only works on the primary selection, which should
  // be a JGoSubGraph
  void ungroupAction() {
    JGoSelection selection = myView.getSelection();
    JGoObject primary = selection.getPrimarySelection();
    if (primary == null) return;
    if (!(primary instanceof JGoArea)) return;
    JGoLayer layer = primary.getLayer();
    if (layer == null) return;
    startTransaction();
    JGoArea area = (JGoArea)primary;
    ArrayList arr = null;

    // special cases for subgraphs--don't promote Handle or Label
    // or other special subgraph children as stand-alone objects
    if (area instanceof JGoSubGraph) {
      JGoSubGraph sg = (JGoSubGraph)area;
      ArrayList children = new ArrayList();
      JGoListPosition pos = sg.getFirstObjectPos();
      while (pos != null) {
        JGoObject obj = sg.getObjectAtPos(pos);
        pos = sg.getNextObjectPosAtTop(pos);
        // check for special subgraph children
        if (obj != sg.getHandle() && obj != sg.getLabel() &&
          !(obj instanceof JGoPort) && obj != sg.getCollapsedObject())
          children.add(obj);
      }
      arr = layer.addCollection(children, true, myMainLayer);
    } else {
      arr = layer.addCollection(area, false, myMainLayer);
    }

    for (int i = 0; i < arr.size(); i++) {
      JGoObject obj = (JGoObject)arr.get(i);
      // update the selectability and visibility, just in case
      obj.setSelectable(true);
      obj.setVisible(true);
      // update the selection too, for the user's convenience
      selection.extendSelection(obj);
    }
    
    // delete the area
    layer.removeObject(area);
    endTransaction("ungroup");
  }

  
  // basic components for this app
  protected Demo1View myView;
  protected JGoDocument myDoc;
  protected JGoLayer myMainLayer;
  protected JGoLayer myForegroundLayer;
  protected JGoLayer myBackgroundLayer;
  protected Point myDefaultLocation = new Point(10, 10);
  protected boolean myRandom = true;

  protected JGoPalette myPalette;
  protected org.eclipse.swt.widgets.List myList;
  protected JGoOverview myOverview;

  protected Shell myShell = null;
}
