/*
 *  Copyright  Northwoods Software Corporation, 1998-2007. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 *
 */

package com.nwoods.jgo.examples.demo1SWT;

import java.util.*;
import org.eclipse.swt.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.custom.TableEditor;
import java.lang.reflect.*;
import com.nwoods.jgo.*;

public class Inspector
{
  public static class InspectorDisposeListener extends Object implements DisposeListener {
    public void widgetDisposed(DisposeEvent e) {
      myDialog = null;
      myTable = null;
      myModel = null;
    }

  }

  public static class InspectorTableModel extends Object
  {
    public InspectorTableModel() {}

    public Object getObject() { return myObject; }

    public void setObject(Object obj) {
      myObject = obj;
      myGetters = new ArrayList();
      mySetters = new ArrayList();
      if (myObject != null) {
        addMethods(myObject.getClass());
      }
    }

    public String getColumnName(int col) {
      if (col == 0) return "Property";
      else return "Value";
    }

    public int getColumnCount() { return 2; }

    public int getRowCount() {
      return myGetters.size();
    }

    public Object getValueAt(int row, int col) {
      Method m = (Method)myGetters.get(row);
      if (col == 0) {
        return getPropertyName(m);
      } else {
        try {
          Object val = m.invoke(myObject, new Object[] {});
          if (val instanceof Point) {
            Point p = (Point)val;
            return Integer.toString(p.x) + "," + Integer.toString(p.y);
          } else if (val instanceof Dimension) {
            Dimension d = (Dimension)val;
            return Integer.toString(d.width) + "x" + Integer.toString(d.height);
          } else if (val instanceof Rectangle) {
            Rectangle r = (Rectangle)val;
            return Integer.toString(r.x) + "," + Integer.toString(r.y) + ";" + Integer.toString(r.width) + "x" + Integer.toString(r.height);
          }
          return val;
        } catch (Exception x) {
          return x.toString();
        }
      }
    }

    public boolean isCellEditable(int row, int col) {
      if (col == 1 && mySetters.get(row) != null) {
        Class type = myObject.getClass();
        if (type != Color.class && type != Insets.class &&
            type != JGoPen.class && type != JGoBrush.class)
          return true;
      }
      return false;
    }

    public void setValueAt(Object value, int row, int col) {
      Method setter = (Method)mySetters.get(row);
      if (setter == null) return;
      Class[] paramtypes = setter.getParameterTypes();
      if (paramtypes.length == 0) return;
      Class type = paramtypes[0];
      if (value instanceof String) {
        String v = (String)value;
        if (type == String.class) {
          invokeSetter(setter, v);
        } else if (type == boolean.class) {
          Boolean newval = Boolean.FALSE;
          if (v.equals("true"))
            newval = Boolean.TRUE;
          invokeSetter(setter, newval);
        } else if (type == int.class) {
          try {
            Integer newval = Integer.valueOf(v);
            invokeSetter(setter, newval);
          } catch (Exception x) {
            // ToDo SSS: Display SWT error message dialog
          }
        } else if (type == float.class) {
          try {
            Float newval = Float.valueOf(v);
            invokeSetter(setter, newval);
          } catch (Exception x) {
          }
        } else if (type == double.class) {
          try {
            Double newval = Double.valueOf(v);
            invokeSetter(setter, newval);
          } catch (Exception x) {
          }
        } else if (type == Point.class) {
          try {
            int comma = v.indexOf(',');
            int x = Integer.parseInt(v.substring(0, comma).trim());
            int y = Integer.parseInt(v.substring(comma+1).trim());
            Point newval = new Point(x, y);
            invokeSetter(setter, newval);
          } catch (Exception x) {
          }
        } else if (type == Dimension.class) {
          try {
            int comma = v.indexOf('x');
            int w = Integer.parseInt(v.substring(0, comma).trim());
            int h = Integer.parseInt(v.substring(comma+1).trim());
            Dimension newval = new Dimension(w, h);
            invokeSetter(setter, newval);
          } catch (Exception x) {

          }
        } else if (type == Rectangle.class) {
          try {
            int comma = v.indexOf(',');
            int x = Integer.parseInt(v.substring(0, comma).trim());
            int semicolon = v.indexOf(';');
            int y = Integer.parseInt(v.substring(comma+1, semicolon).trim());
            int by = v.indexOf('x');
            int w = Integer.parseInt(v.substring(semicolon+1, by).trim());
            int h = Integer.parseInt(v.substring(by+1).trim());
            Point newval = new Point(x, y);
            invokeSetter(setter, newval);
          } catch (Exception x) {
          }
        } else {
        }
      } else {
      }
    }

    private void invokeSetter(Method setter, Object newval) {
      JGoDocument doc = null;
      if (myObject instanceof JGoDocument)
        doc = (JGoDocument)myObject;
      else if (myObject instanceof JGoObject)
        doc = ((JGoObject)myObject).getDocument();
      else if (myObject instanceof JGoLayer)
        doc = ((JGoLayer)myObject).getDocument();
      else if (myObject instanceof JGoView)
        doc = ((JGoView)myObject).getDocument();
      else if (myObject instanceof JGoSelection)
        doc = ((JGoSelection)myObject).getView().getDocument();
      if (doc != null)
        doc.startTransaction();
      try {
        setter.invoke(myObject, new Object[] { newval });
      } catch (Exception x) {
      }
      if (doc != null)
        doc.endTransaction("modified property: " + setter.toString());
    }


    public void mouseClicked(int count, int row, int column) {
      if (count >= 2) {
        Method getter = (Method)myGetters.get(row);
        Class type = getter.getReturnType();
        if (type != String.class && type != boolean.class && type != int.class &&
            type != float.class && type != double.class && type != Class.class &&
            type != Point.class && type != Dimension.class && type != Rectangle.class &&
            type != Color.class && type != Insets.class) {
          Object value = getValueAt(row, 1);
          if (value != null)
            setObject(value);
        }
      }
    }

    public boolean isEditableObject(Object obj) {
      if ((obj instanceof String) ||
        (obj instanceof Boolean) ||
        (obj instanceof Integer) ||
        (obj instanceof Float) ||
        (obj instanceof Double) ||
        (obj instanceof Point) ||
        (obj instanceof Dimension) ||
        (obj instanceof Rectangle) ||
        (obj instanceof Color) ||
        (obj instanceof Insets)) {
      return true;
    }
    return false;
    }

    private void addMethods(Class c) {
      if (c.getSuperclass() != null)
        addMethods(c.getSuperclass());
      Method[] methods = c.getDeclaredMethods();
      for (int i = 0; i < methods.length; i++) {
        Method m = methods[i];
        if (isDisplayable(m)) {
          myGetters.add(m);
          String setname = "set" + getPropertyName(m);
          try {
            Method setter = myObject.getClass().getMethod(setname, new Class[] { m.getReturnType() });
            mySetters.add(setter);
          } catch (Exception x) {
            mySetters.add(null);
          }
        }
      }
    }

    private String getPropertyName(Method m) {
      String name = m.getName();
      if (name.length() > 3 && name.substring(0, 3).equals("get"))
        return name.substring(3);
      else if (name.length() > 2 && name.substring(0, 2).equals("is"))
        return name.substring(2);
      else return null;
    }

    private boolean isDisplayable(Method m) {
      return m.getParameterTypes().length == 0 &&
             getPropertyName(m) != null &&
             Modifier.isPublic(m.getModifiers());
    }

    private Object myObject = null;
    private ArrayList myGetters = new ArrayList();
    private ArrayList mySetters = new ArrayList();
  }

  public static void inspect(Display display, Shell parentShell, Object obj) {
    try {
      myDisplay = display;
      myParentShell = parentShell;
      if (myDialog == null) {
        myModel = new InspectorTableModel();
        Shell shell = new Shell(myParentShell, SWT.DIALOG_TRIM | SWT.RESIZE);
        shell.setText("Properties");
        shell.setSize(230, 400);
        FillLayout layout = new FillLayout();
        shell.setLayout(layout);
        myDialog = new Composite(shell, SWT.NONE);
        myDialog.setLayout(new FillLayout());
        myDialog.addDisposeListener(new InspectorDisposeListener());

        myTable = new Table(myDialog, SWT.SINGLE | SWT.FULL_SELECTION);

        editor = new TableEditor (myTable);
        myTable.addSelectionListener(new SelectionAdapter() {
          public void widgetSelected(SelectionEvent e) {
            // Clean up any previous editor control
            if (editor != null) {
              Control oldEditor = editor.getEditor();
              if (oldEditor != null)
                oldEditor.dispose();
            }

            // Identify the selected row
            int index = myTable.getSelectionIndex();
            if (index == -1)
              return;
            TableItem item = myTable.getItem(index);

            //The text editor must have the same size as the cell and must
            //not be any smaller than 50 pixels.
            editor.horizontalAlignment = SWT.LEFT;
            editor.grabHorizontal = true;
            editor.minimumWidth = 50;

            // See if the selected row value is editable as a text string
            // If so, use a Text object as the TableEditor editor.
            // Otherwise, assume it is an object and create a Button object as
            // the TableEditor editor.  Pressing the button will make the object
            // the current object in the Inspector.
            Integer intRow = (Integer) item.getData();
            Object obj = myModel.getValueAt(intRow.intValue(), 1);
            if (!myModel.isEditableObject(obj)) {
              Button button = new Button(myTable, SWT.NONE);
              String curVal = item.getText(1);
              button.setText(curVal);
              editor.setEditor(button, item, 1);
              button.addSelectionListener(new SelectionAdapter() {
                public void widgetSelected(SelectionEvent e) {
                  // Identify the selected row
                  int index = myTable.getSelectionIndex();
                  if (index == -1)
                    return;
                  TableItem item = myTable.getItem(index);
                  Integer intRow = (Integer) item.getData();
                  Object obj = myModel.getValueAt(intRow.intValue(), 1);
                  if (!myModel.isEditableObject(obj)) {
                    myModel.setObject(obj);
                    refresh();
                  }
                }
              });
            }
            else {
              // The control that will be the editor must be a child of the Table
              Text text = new Text(myTable, SWT.NONE);
              String curVal = item.getText(1);
              text.setText(curVal);
              text.addFocusListener(new FocusAdapter() {
                public void focusGained(FocusEvent e) {
                }

                public void focusLost(FocusEvent e) {
                  Text text = (Text) e.getSource();
                  // Identify the selected row
                  int index = myTable.getSelectionIndex();
                  if (index == -1)
                    return;
                  TableItem item = myTable.getItem(index);
                  item.setText(1, text.getText());
                  Integer intRow = (Integer) item.getData();
                  Object obj = myModel.getValueAt(intRow.intValue(), 1);
                  if (myModel.isEditableObject(obj)) {
                    // Now, actually set the new property value in the object
                    myModel.setValueAt(text.getText(), intRow.intValue(), 1);
                  }
                }
              });


              // Open the text editor in the second column of the selected row.
              editor.setEditor(text, item, 1);
              editor.getEditor().addKeyListener(new KeyAdapter() {
                public void keyPressed(KeyEvent e) {
                  // If the Enter key is pressed, update the property value
                  if (e.character == 13) {
                    Text text = (Text) e.getSource();
                    // Identify the selected row
                    int index = myTable.getSelectionIndex();
                    if (index == -1)
                      return;
                    TableItem item = myTable.getItem(index);
                    item.setText(1, text.getText());
                    Integer intRow = (Integer) item.getData();
                    Object obj = myModel.getValueAt(intRow.intValue(), 1);
                    if (! (obj instanceof String) && ! (obj instanceof Boolean)) {
                      myModel.setObject(obj);
                      refresh();
                    }
                    else {
                      // Now, actually set the new property value in the object
                      myModel.setValueAt(text.getText(), intRow.intValue(), 1);
                    }
                  }
                }
              });

              // Assign focus to the text control
              text.setFocus();
            }
          }
        });
        // Finally, create the table columns, call setObject() to create the
        // list of methods, and call refresh() to actually populate the table.
        TableColumn column = new TableColumn(myTable, SWT.NONE);
        column.setText("Property");
        column.setWidth(100);
        column = new TableColumn(myTable, SWT.NONE);
        column.setText("Value");
        column.setWidth(100);
        myTable.setHeaderVisible(true);
        shell.open();
      }
      myModel.setObject(obj);
      refresh();

      }
      catch (Exception e) {
        System.err.println(e);
        e.printStackTrace();
      }
    }

    public static void refresh() {
      // Clean up any previous editor control
      if (editor != null) {
        Control oldEditor = editor.getEditor();
        if (oldEditor != null)
          oldEditor.dispose();
      }
      // Create new table contents
      if (myTable != null)
        myTable.removeAll();
      if (myModel != null) {
        String[] itemText = new String[2];
        int currentRow  = 0;
        for (int i = 0; i < myModel.getRowCount(); i++) {
          try {
            String propName = (String)myModel.getValueAt(i, 0);
            Object propValue = myModel.getValueAt(i, 1);
            itemText[0] = propName;
            itemText[1] = propValue.toString();
            currentRow++;
            TableItem item = new TableItem(myTable, SWT.NONE);
            item.setData(new Integer(i));
            item.setText(itemText);
          }
          catch (Exception e) {
          }
        }
      }
    }

  private static InspectorTableModel myModel = null;
  public static Composite getInspector() { return myDialog; }

  private static Composite myDialog = null;
  private static Table myTable = null;
  private static TableEditor editor = null;

  private static Display myDisplay = null;
  private static Shell myParentShell = null;
}
