/*
 *  Copyright  Northwoods Software Corporation, 2000-2007. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 *
 */

package com.nwoods.jgo.examples.familySWT;

import org.eclipse.swt.*;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.custom.SashForm;
import com.nwoods.jgo.*;


/**
 * This simple app show a part of the family tree of a famous family.
 * People are represented by color-coded nodes.  Marriages are represented
 * by thick green links between two nodes.  Child-parent relationships are
 * represented by links from the child to the particular marriage linking
 * the biological mother and biological father.  Tooltips present additional
 * information, if available, for each person.
 * <p>
 * Because this is an example, commands are limited to direct manipulation
 * and keyboard commands, the genealogical database is kept very simple, and
 * the automatic layout of the nodes in the document is moderately simple.
 */
public class FamilyTree
{
  public FamilyTree(Shell shell)
  {
    SashForm form = new SashForm(shell, SWT.HORIZONTAL);
    // Create the JGo model for the family tree view
    myDoc = new FamilyTreeDoc();
    // Create the main graphics window
    myView = new JGoView(form, SWT.RIGHT | SWT.V_SCROLL | SWT.H_SCROLL, myDoc);

    // Process KeyEvents from the JGoView--handle the Delete key.
    //
    // Normally one overrides JGoView.onKeyEvent() to make use of the
    // default behavior supplied by JGoView.  Using a listener,
    // as this example does, may lead to duplicate and/or conflicting
    // actions if the same key is handled, and does not allow
    // inhibiting the default behavior.  For this example program,
    // we assume the default behavior is OK, and we don't want to
    // bother creating a subclass of JGoView yet.
    myView.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent evt) {
        boolean ctrl = ((evt.stateMask & SWT.CTRL) != 0);
        boolean shift = ((evt.stateMask & SWT.SHIFT) != 0);
        char ch = evt.character;
        int code = evt.keyCode;
        if (code == SWT.DEL) {
          // delete all selected PersonNodes
          myView.deleteSelection();
        } else if (code == SWT.INSERT) {
          // delete everything and then start over
          myDoc.deleteContents();
          myDoc.initNodes();
        } else if (ctrl && ch == 12) {  // Ctrl-L
          // re-layout all the nodes
          myView.getSelection().clearSelectionHandles(null);
          myDoc.layoutNodes();
          myView.getSelection().restoreSelectionHandles(null);
        } else if (ctrl && ch == 17) {  // Ctrl-Q
          System.exit(0);
        } else if (ctrl && ch == 16) {  // Ctrl-P
          myView.print();
        }
      }
    });
  }

  public void init()
  {
    // create the initial set of PersonNodes from the database
    myDoc.initNodes();
  }

  static public void main(String args[])
  {
    try {
      Display display = new Display();
      Shell shell = new Shell(display);
      shell.setText("The Tudors");
      FillLayout layout = new FillLayout();
      shell.setLayout(layout);

      FamilyTree app = new FamilyTree(shell);

      shell.open();
      app.init();
      while (!shell.isDisposed()) {
        if (!display.readAndDispatch())
          display.sleep();
      }
    } catch (Throwable t) {
      System.err.println(t);
      t.printStackTrace();
      System.exit(1);
    }
  }

  // State
  protected JGoView myView;
  protected FamilyTreeDoc myDoc;
}
