/*
 *  Copyright  Northwoods Software Corporation, 2000-2007. All Rights
 *  Reserved.
 *
 *  Restricted Rights: Use, duplication, or disclosure by the U.S.
 *  Government is subject to restrictions as set forth in subparagraph
 *  (c) (1) (ii) of DFARS 252.227-7013, or in FAR 52.227-19, or in FAR
 *  52.227-14 Alt. III, as applicable.
 *
 */

package com.nwoods.jgo.examples.processorSWT;

import org.eclipse.swt.*;
import org.eclipse.swt.custom.*;
import org.eclipse.swt.dnd.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;
import java.util.HashMap;
import java.net.URL;
import com.nwoods.jgo.*;
import com.nwoods.jgo.examples.*;


/**
 * This example app is a simple process flow editor
 */
public class Processor {
  public Processor(Shell shell) {
    try {
      myShell = shell;

      GridLayout layout = new GridLayout(1, true);
      shell.setLayout(layout);

      Menu menubar = new Menu(shell, SWT.BAR);
      shell.setMenuBar(menubar);

      ToolBar toolbar = new ToolBar(shell, SWT.HORIZONTAL | SWT.WRAP);

      SashForm form = new SashForm(shell, SWT.HORIZONTAL);
      GridData data = new GridData(GridData.FILL_BOTH);
      form.setLayoutData(data);

      myPalette = new JGoPalette(form, SWT.V_SCROLL | SWT.H_SCROLL);

      myProcessView = new ProcessView(form, SWT.V_SCROLL | SWT.H_SCROLL);
      myProcessView.initialize(this);
      myProcessView.setFocus();

      int[] weights = new int[2];
      weights[0] = 15;
      weights[1] = 85;
      form.setWeights(weights);

      initMenus(menubar);
      initToolbar(toolbar);
      initPalette();

    } catch (Throwable t) {
      System.err.println(t);
      t.printStackTrace();
    }
  }


  //==============================================================
  // Define all the command actions
  //==============================================================

  void initMenus(Menu mb)
  {
    // Create File menu
    MenuItem miFile = new MenuItem(mb, SWT.CASCADE);
    miFile.setText("&File");
    Menu fileMenu = new Menu(miFile);
    miFile.setMenu(fileMenu);

    // Create File menu items
    MenuItem miFileOpen = new MenuItem(fileMenu, SWT.NONE);
    miFileOpen.setText("&Open\tCtrl-O");
    miFileOpen.setAccelerator(SWT.CONTROL | 'O');
    miFileOpen.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.setDocument(ProcessDocument.open(myShell)); }
    });

    MenuItem miFileSave = new MenuItem(fileMenu, SWT.NONE);
    miFileSave.setText("&Save\tCtrl-S");
    miFileSave.setAccelerator(SWT.CONTROL | 'S');
    miFileSave.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.getDoc().save(); }
    });

    MenuItem miFileSaveAs = new MenuItem(fileMenu, SWT.NONE);
    miFileSaveAs.setText("Save &As");
    miFileSaveAs.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.getDoc().saveAs(); }
    });
    MenuItem miFilePrint = new MenuItem(fileMenu, SWT.NONE);
    miFilePrint.setText("&Print\tCtrl-P");
    miFilePrint.setAccelerator(SWT.CONTROL | 'P');
    miFilePrint.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.print(); }
    });
    new MenuItem(fileMenu, SWT.SEPARATOR);
    MenuItem miFileExit = new MenuItem(fileMenu, SWT.NONE);
    miFileExit.setText("E&xit");
    miFileExit.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { System.exit(0); }
    });

    // Create Edit menu
    MenuItem miEdit = new MenuItem(mb, SWT.CASCADE);
    miEdit.setText("&Edit");
    Menu editMenu = new Menu(miEdit);
    miEdit.setMenu(editMenu);

    // Create Edit menu items
    MenuItem miCut = new MenuItem(editMenu, SWT.NONE);
    miCut.setText("Cu&t\tCtrl-X");
    miCut.setAccelerator(SWT.CONTROL | 'X');
    miCut.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.cut(); }
    });
    MenuItem miCopy = new MenuItem(editMenu, SWT.NONE);
    miCopy.setText("&Copy\tCtrl-C");
    miCopy.setAccelerator(SWT.CONTROL | 'C');
    miCopy.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.copy(); }
    });
    MenuItem miPaste = new MenuItem(editMenu, SWT.NONE);
    miPaste.setText("&Paste\tCtrl-V");
    miPaste.setAccelerator(SWT.CONTROL | 'V');
    miPaste.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.paste(); }
    });

    MenuItem miDelete = new MenuItem(editMenu, SWT.NONE);
    miDelete.setText("&Delete\tDel");
    miDelete.setAccelerator(SWT.DEL);
    miDelete.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.deleteSelection(); }
    });

    MenuItem miSelectAll = new MenuItem(editMenu, SWT.NONE);
    miSelectAll.setText("Select All\tCtrl-A");
    miSelectAll.setAccelerator(SWT.CONTROL | 'A');
    miSelectAll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.selectAll(); }
    });

    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miUndo = new MenuItem(editMenu, SWT.NONE);
    miUndo.setText("&Undo\tCtrl-Z");
    miUndo.setAccelerator(SWT.CONTROL | 'Z');
    miUndo.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.getDocument().undo(); }
    });
    MenuItem miRedo = new MenuItem(editMenu, SWT.NONE);
    miRedo.setText("&Redo\tCtrl-Y");
    miRedo.setAccelerator(SWT.CONTROL | 'Y');
    miRedo.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.getDocument().redo(); }
    });

    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miDrawRoutedLink = new MenuItem(editMenu, SWT.NONE);
    miDrawRoutedLink.setText("Draw Routed Link");
    miDrawRoutedLink.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.editDrawRoutedLink(); }
    });
    MenuItem miMovePort = new MenuItem(editMenu, SWT.NONE);
    miMovePort.setText("Move Port");
    miMovePort.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.editMovePort(); }
    });

    new MenuItem(editMenu, SWT.SEPARATOR);

    MenuItem miProperties = new MenuItem(editMenu, SWT.NONE);
    miProperties.setText("Properties");
    miProperties.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.editObjectProperties(); }
    });

    // Create View menu
    MenuItem miView = new MenuItem(mb, SWT.CASCADE);
    miView.setText("&View");
    Menu viewMenu = new Menu(miView);
    miView.setMenu(viewMenu);

    // Create View menu items
    MenuItem miZoomIn = new MenuItem(viewMenu, SWT.NONE);
    miZoomIn.setText("Zoom In\tCtrl-F6");
    miZoomIn.setAccelerator(SWT.CONTROL | SWT.F6);
    miZoomIn.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.zoomIn(); }
    });
    MenuItem miZoomOut = new MenuItem(viewMenu, SWT.NONE);
    miZoomOut.setText("Zoom Out\tShift-F6");
    miZoomOut.setAccelerator(SWT.SHIFT | SWT.F6);
    miZoomOut.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.zoomOut(); }
    });
    MenuItem miZoomNormal = new MenuItem(viewMenu, SWT.NONE);
    miZoomNormal.setText("Zoom Normal\tCtrl-Shift-F6");
    miZoomNormal.setAccelerator(SWT.CONTROL | SWT.SHIFT | SWT.F6);
    miZoomNormal.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.zoomNormal(); }
    });
    MenuItem miZoomToFit = new MenuItem(viewMenu, SWT.NONE);
    miZoomToFit.setText("Zoom To Fit\tF6");
    miZoomToFit.setAccelerator(SWT.F6);
    miZoomToFit.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.zoomToFit(); }
    });

    new MenuItem(viewMenu, SWT.SEPARATOR);

    MenuItem miLinksJumpOver = new MenuItem(viewMenu, SWT.CHECK);
    miLinksJumpOver.setText("LinksJumpOver\tCtrl-F6");
    miLinksJumpOver.setAccelerator(SWT.CONTROL | SWT.F6);
    miLinksJumpOver.setSelection(myProcessView.getDoc().getLinksJumpOver());
    miLinksJumpOver.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) {
        myProcessView.getDoc().setLinksJumpOver(!myProcessView.getDoc().getLinksJumpOver());
      }
    });

    // Create Insert menu
    MenuItem miInsert = new MenuItem(mb, SWT.CASCADE);
    miInsert.setText("&Insert");
    Menu insertMenu = new Menu(miInsert);
    miInsert.setMenu(insertMenu);

    // Create Insert menu items
    MenuItem miInsertComment = new MenuItem(insertMenu, SWT.NONE);
    miInsertComment.setText("Comment");
    miInsertComment.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.insertComment(); }
    });
    MenuItem miInsertStart = new MenuItem(insertMenu, SWT.NONE);
    miInsertStart.setText("Start");
    miInsertStart.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.insertInput(); }
    });
    MenuItem miInsertFinish = new MenuItem(insertMenu, SWT.NONE);
    miInsertFinish.setText("Finish");
    miInsertFinish.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.insertOutput(); }
    });
    MenuItem miInsertActivity = new MenuItem(insertMenu, SWT.NONE);
    miInsertActivity.setText("Activity");
    miInsertActivity.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event) { myProcessView.insertActivity(); }
    });
  }

  protected void initToolbar(ToolBar bar)
  {
    ToolItem item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Open");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/open.gif")));
    item.setToolTipText("Load a work-flow from a file");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.setDocument(ProcessDocument.open(myShell)); }
    });

    item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Save");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/save.gif")));
    item.setToolTipText("Save a work-flow to a file");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.getDoc().save(); }
    });

    item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Cut");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/remove.gif")));
    item.setToolTipText("Cut to clipboard");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.cut(); }
    });

    item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Copy");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/copy.gif")));
    item.setToolTipText("Copy to clipboard");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.copy(); }
    });

    item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Paste");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/paste.gif")));
    item.setToolTipText("Paste from clipboard");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.paste(); }
    });

    item = new ToolItem(bar, SWT.PUSH);
    //item.setText("Print");
    item.setImage(new Image(myShell.getDisplay(), Processor.class.getResourceAsStream("images/print.gif")));
    item.setToolTipText("Print");
    item.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) { myProcessView.print(); }
    });
  }

  void initPalette()
  {
    JGoDocument doc = myPalette.getDocument();

    Comment cmnt = new Comment("a comment");
    cmnt.setEditable(true);
    doc.addObjectAtTail(cmnt);

    ActivityNode snode;
    for (int i = 0; i < 8; i++) {
      ActivityNode n = makePaletteNode(i, -1);
      doc.addObjectAtTail(n);
    }
  }

  ActivityNode makePaletteNode(int acttype, int id)
  {
    ActivityNode snode = new ActivityNode();
    JGoImage nodeicon = new JGoImage(new Rectangle(0,0,40,40));
    nodeicon.loadImage(Processor.class.getResource("images/doc.gif"), true);
    snode.initialize(acttype, id);
    snode.addScatteredPorts((int)(Math.random()*5)+1);
    snode.getLabel().setSelectable(false);
    return snode;
  }

  static private Processor theApp = null;
  static public Processor getApp() { return theApp; }

  public Shell getShell() { return myShell; }

  static public void main(String args[]) {
    try {
      Display display = new Display();
      Shell shell = new Shell(display);
      shell.setText("Process Flow Editor");

      theApp = new Processor(shell);

      MultiPortNodePort.FULL = JGoBrush.green;

      shell.open();
      while (!shell.isDisposed()) {
        if (!display.readAndDispatch())
          display.sleep();
      }
    } catch (Throwable t) {
      System.err.println(t);
      t.printStackTrace();
      System.exit(1);
    }
  }


  void editProcessProperties()
  {
    ProcessDocument doc = myProcessView.getDoc();
    doc.startTransaction();
//???    new ProcessDialog(getShell(), doc).setVisible(true);
    doc.endTransaction("Process Properties");
  }

  // State
  protected ProcessView myProcessView;
  protected JGoPalette myPalette;
  protected Shell myShell;
}
